import { Box, Button, Container, Grid, TextField } from '@material-ui/core';
import { makeStyles } from '@material-ui/core/styles';
import { ArrowBackIos, DeleteForever, Save } from '@material-ui/icons';
import React, { FormEvent } from 'react';
import { useSelector } from 'react-redux';
import { ADDRESS_REQUIRED, CITY_REQUIRED, COMMERCIAL_NAME_REQUIRED, EMAIL_REQUIRED, INVALID_ACCOUNT, INVALID_EMAIL, NAME_LENGTH, NAME_REQUIRED, NIF_REQUIRED, PHONE_REQUIRED, POSTAL_CODE_LENGTH, POSTAL_CODE_REQUIRED, SURNAMES_REQUIRED } from '../../constants/errors';
import { getError, hasError } from '../../lib/errors';
import { AppState } from '../../redux/reducers';
import { ErrorsState } from '../../redux/reducers/errors';
import { UiState } from '../../redux/reducers/ui';
import CitySelector from '../common/CitySelector';
import CustomBackdrop from '../common/CustomBackdrop';

type Props = {
  data: any;
  setData: Function;
  newClient: boolean;
  submit: (e: FormEvent<HTMLFormElement>) => void;
  cancel: () => void;
  deleteClient: () => void;
}

const useStyles = makeStyles((theme) => ({
  paper: {
    marginTop: theme.spacing(4),
    display: 'flex',
    flexDirection: 'column',
    alignItems: 'center',
  },
  form: {
    width: '100%', // Fix IE 11 issue.
    marginTop: theme.spacing(1),
  },
  submit: {
    margin: theme.spacing(5, 0, 2),
  },
  centerText: {
    textAlign: "center"
  },
  spacer: {
    width: '100%',
    padding: theme.spacing(1),
  },
}));

const ClientForm = ({ data, setData, newClient, submit, cancel, deleteClient }: Props) => {
  const classes = useStyles();
  const errors: ErrorsState = useSelector<AppState, any>(state => state.errors);
  const { isLoading, isSaving, isDeleting }: UiState = useSelector<AppState, any>(state => state.ui);

  return (
    <Container component="main" maxWidth="md">
      <div className={classes.paper}>
        <fieldset style={{ border: "none" }} disabled={isLoading || isSaving || isDeleting}>
          <form onSubmit={submit}>
            <Grid container spacing={3}>
              <Grid item xs={12} sm={4}>
                <TextField
                  variant="outlined"
                  size="small"
                  fullWidth
                  id="name"
                  label="Nom"
                  name="name"
                  autoFocus
                  InputLabelProps={{ shrink: true }}
                  error={hasError(errors, NAME_LENGTH) || hasError(errors, NAME_REQUIRED)}
                  helperText={getError(errors, NAME_LENGTH) || getError(errors, NAME_REQUIRED)}
                  value={data.name}
                  onChange={e => setData({ ...data, name: e.target.value })}
                />
              </Grid>
              <Grid item xs={12} sm={8}>
                <TextField
                  variant="outlined"
                  size="small"
                  fullWidth
                  id="surnames"
                  label="Cognoms"
                  name="surnames"
                  InputLabelProps={{ shrink: true }}
                  error={hasError(errors, SURNAMES_REQUIRED)}
                  helperText={getError(errors, SURNAMES_REQUIRED)}
                  value={data.surnames}
                  onChange={e => setData({ ...data, surnames: e.target.value })}
                />
              </Grid>

              <Grid item xs={12} sm={9}>
                <TextField
                  variant="outlined"
                  size="small"
                  fullWidth
                  id="commercialName"
                  label="Nom comercial"
                  name="commercialName"
                  InputLabelProps={{ shrink: true }}
                  error={hasError(errors, COMMERCIAL_NAME_REQUIRED)}
                  helperText={getError(errors, COMMERCIAL_NAME_REQUIRED)}
                  value={data.commercialName}
                  onChange={e => setData({ ...data, commercialName: e.target.value })}
                />
              </Grid>
              <Grid item xs={12} sm={3}>
                <TextField
                  variant="outlined"
                  size="small"
                  fullWidth
                  required
                  id="nif"
                  label="NIF"
                  name="nif"
                  InputLabelProps={{ shrink: true }}
                  error={hasError(errors, NIF_REQUIRED)}
                  helperText={getError(errors, NIF_REQUIRED)}
                  value={data.nif}
                  onChange={e => setData({ ...data, nif: e.target.value })}
                />
              </Grid>

              <Grid item xs={12} sm={3}>
                <TextField
                  variant="outlined"
                  size="small"
                  fullWidth
                  id="phone"
                  label="Telèfon"
                  name="phone"
                  InputLabelProps={{ shrink: true }}
                  error={hasError(errors, PHONE_REQUIRED)}
                  helperText={getError(errors, PHONE_REQUIRED)}
                  value={data.phone}
                  onChange={e => setData({ ...data, phone: e.target.value })}
                />
              </Grid>
              <Grid item xs={12} sm={9}>
                <TextField
                  variant="outlined"
                  size="small"
                  fullWidth
                  required
                  id="address"
                  label="Adreça"
                  name="address"
                  InputLabelProps={{ shrink: true }}
                  error={hasError(errors, ADDRESS_REQUIRED)}
                  helperText={getError(errors, ADDRESS_REQUIRED)}
                  value={data.address}
                  onChange={e => setData({ ...data, address: e.target.value })}
                />
              </Grid>

              <Grid item xs={12} sm={4}>
                <div style={{ display: 'flex' }}>
                  <CitySelector
                    setData={(city: any) => setData({ ...data, postalcode: city.code, city: city.city })}
                  />
                  <TextField
                    variant="outlined"
                    size="small"
                    fullWidth
                    required
                    id="postalcode"
                    label="Codi postal"
                    name="postalcode"
                    InputLabelProps={{ shrink: true }}
                    error={hasError(errors, POSTAL_CODE_REQUIRED) || hasError(errors, POSTAL_CODE_LENGTH)}
                    helperText={getError(errors, POSTAL_CODE_REQUIRED || hasError(errors, POSTAL_CODE_LENGTH))}
                    value={data.postalcode}
                    onChange={e => setData({ ...data, postalcode: e.target.value })}
                  />
                </div>
              </Grid>
              <Grid item xs={12} sm={8}>
                <TextField
                  variant="outlined"
                  size="small"
                  fullWidth
                  required
                  id="city"
                  label="Municipi"
                  name="city"
                  InputLabelProps={{ shrink: true }}
                  error={hasError(errors, CITY_REQUIRED)}
                  helperText={getError(errors, CITY_REQUIRED)}
                  value={data.city}
                  onChange={e => setData({ ...data, city: e.target.value })}
                />
              </Grid>

              <Grid item xs={12} sm={6}>
                <TextField
                  variant="outlined"
                  size="small"
                  fullWidth
                  required
                  id="email"
                  label="Correu electrònic"
                  name="email"
                  InputLabelProps={{ shrink: true }}
                  error={hasError(errors, EMAIL_REQUIRED) || hasError(errors, INVALID_EMAIL)}
                  helperText={getError(errors, EMAIL_REQUIRED) || getError(errors, INVALID_EMAIL)}
                  value={data.email}
                  onChange={e => setData({ ...data, email: e.target.value })}
                />
              </Grid>
              <Grid item xs={12} sm={6}>
                <TextField
                  variant="outlined"
                  size="small"
                  fullWidth
                  id="account"
                  label="Compte corrent"
                  name="account"
                  InputLabelProps={{ shrink: true }}
                  error={hasError(errors, INVALID_ACCOUNT)}
                  helperText={getError(errors, INVALID_ACCOUNT)}
                  value={data.account}
                  onChange={e => setData({ ...data, account: e.target.value })}
                />
              </Grid>

              <Box className={classes.spacer} />
              <Grid container spacing={3}>
                <Grid item xs={newClient ? 6 : 4}>
                  <Button
                    type="submit"
                    fullWidth
                    variant="contained"
                    color="primary"
                    startIcon={<Save />}
                  >
                    {isSaving ? "Desant" : "Desa"}
                  </Button>
                </Grid>
                <Grid item xs={newClient ? 6 : 4}>
                  <Button
                    type="button"
                    fullWidth
                    variant="contained"
                    startIcon={<ArrowBackIos />}
                    onClick={cancel}
                  >
                    Cancel·la
                  </Button>
                </Grid>
                {!newClient &&
                  <Grid item xs={4}>
                    <Button
                      type="button"
                      fullWidth
                      variant="contained"
                      color="secondary"
                      startIcon={<DeleteForever />}
                      onClick={deleteClient}
                    >
                      {isDeleting ? "Eliminant" : "Elimina"}
                    </Button>
                  </Grid>
                }
              </Grid>

            </Grid>
          </form>
        </fieldset>
        <CustomBackdrop open={(isLoading || isSaving || isDeleting)} />
      </div>
    </Container>
  )
}

export default ClientForm
