import DateFnsUtils from '@date-io/date-fns';
import { Box, Button, Checkbox, Container, FormControl, FormControlLabel, FormGroup, FormHelperText, Grid, InputLabel, MenuItem, Select, TextField, Typography } from '@material-ui/core';
import { makeStyles } from '@material-ui/core/styles';
import { KeyboardDatePicker, MuiPickersUtilsProvider } from '@material-ui/pickers';
import React from 'react';
import { useSelector } from 'react-redux';
import { CLIENT_REQUIRED, INVALID_DATE, INVOICE_NAME_LENGTH, INVOICE_NAME_REQUIRED, INVOICE_RETENTION, INVOICE_RETENTION_REQUIRED } from '../../constants/errors';
import { getError, hasError } from '../../lib/errors';
import { AppState } from '../../redux/reducers';
import { ErrorsState } from '../../redux/reducers/errors';
import { UiState } from '../../redux/reducers/ui';
import CustomBackdrop from '../common/CustomBackdrop';
import LinesList from '../Lines/LinesList';
import InvoiceFormButtons from './InvoiceFormButtons';
import InvoiceSummary from './InvoiceSummary';

type Props = {
  data: any;
  setData: any;
  clients: any[];
  newInvoice: boolean;
  submit: (e: any) => void;
  emitInvoice: () => void;
  cancelInvoice: () => void;
  printInvoice: () => void;
  cancel: () => void;
  deleteInvoice: () => void;
  addLine: () => void;
}

const useStyles = makeStyles((theme) => ({
  paper: {
    marginTop: theme.spacing(4),
    display: 'flex',
    flexDirection: 'column',
    alignItems: 'center',
  },
  avatar: {
    margin: theme.spacing(1),
    backgroundColor: theme.palette.secondary.main,
  },
  form: {
    width: '100%', // Fix IE 11 issue.
    marginTop: theme.spacing(1),
  },
  submit: {
    margin: theme.spacing(5, 0, 2),
  },
  centerText: {
    textAlign: "center"
  },
  disabledOption: {
    color: 'grey',
    fontStyle: 'italic',
  }
}));

const InvoiceForm = ({ data, setData, clients, newInvoice, submit, emitInvoice, cancelInvoice, printInvoice, cancel, deleteInvoice, addLine }: Props) => {
  const classes = useStyles();
  const errors: ErrorsState = useSelector<AppState, any>(state => state.errors);
  const { isLoading, isSaving, isEmitting, isCancelling, isDeleting }: UiState = useSelector<AppState, any>(state => state.ui);

  return (
    <Container component="main" maxWidth="md">
      <div className={classes.paper}>
        <fieldset disabled={isLoading || isSaving || isEmitting || isCancelling || isDeleting} style={{ border: "none" }}>
          <form onSubmit={submit} className={classes.form}>
            <Grid container spacing={3}>

              <Grid item xs={12}>
                <MuiPickersUtilsProvider utils={DateFnsUtils}>
                  <KeyboardDatePicker
                    disableToolbar
                    variant="inline"
                    format="dd/MM/yyyy"
                    id="date"
                    label="Data"
                    value={data.date}
                    onChange={e => setData({ ...data, date: e })}
                    disabled={data.emitted || data.cancelled}
                    KeyboardButtonProps={{
                      'aria-label': 'change date',
                    }}
                    error={hasError(errors, INVALID_DATE)}
                    helperText={getError(errors, INVALID_DATE)}
                  />
                </MuiPickersUtilsProvider>
              </Grid>

              <Grid item xs={12}>
                <TextField
                  variant="outlined"
                  fullWidth
                  id="name"
                  label="Concepte"
                  name="name"
                  autoFocus
                  InputLabelProps={{ shrink: true }}
                  error={hasError(errors, INVOICE_NAME_REQUIRED || hasError(errors, INVOICE_NAME_LENGTH))}
                  helperText={getError(errors, INVOICE_NAME_REQUIRED) || getError(errors, INVOICE_NAME_LENGTH)}
                  value={data.name}
                  onChange={e => setData({ ...data, name: e.target.value })}
                  disabled={data.emitted || data.cancelled}
                />
              </Grid>

              <Grid item xs={12} sm={10}>
                <FormControl variant="outlined" fullWidth error={hasError(errors, CLIENT_REQUIRED)}>
                  <InputLabel id="demo-simple-select-outlined-label">Client</InputLabel>
                  <Select
                    labelId="demo-simple-select-outlined-label"
                    id="client"
                    fullWidth
                    value={data.client || "0"}
                    onChange={e => setData({ ...data, client: e.target.value })} disabled={!clients.length || data.emitted || data.cancelled}
                    label="Client"
                  >
                    <MenuItem value="0"><span className={classes.disabledOption}>Escull un client</span></MenuItem>
                    {clients.map((client: any) =>
                      <MenuItem value={client._id} key={client._id} >
                        {client.commercialName ? client.commercialName : `${client.surnames}, ${client.name}`}
                      </MenuItem>)}
                  </Select>
                  <FormHelperText>{getError(errors, CLIENT_REQUIRED)}</FormHelperText>
                </FormControl>
              </Grid>

              <Grid item xs={12} sm={2}>
                <TextField
                  type="number"
                  variant="outlined"
                  fullWidth
                  id="ret"
                  label="Retenció"
                  name="ret"
                  InputLabelProps={{ shrink: true }}
                  error={hasError(errors, INVOICE_RETENTION_REQUIRED) || hasError(errors, INVOICE_RETENTION)}
                  helperText={getError(errors, INVOICE_RETENTION_REQUIRED) || getError(errors, INVOICE_RETENTION)}
                  value={data.ret}
                  onChange={e => setData({ ...data, ret: e.target.value })}
                  disabled={data.emitted || data.cancelled}
                />
              </Grid>

              {data._id &&
                <>
                  <Grid item xs={12}>
                    <Typography component="span" variant="h6">Línes de factura</Typography>
                    {!data.emitted && !data.cancelled &&
                      <Box component="span" ml={2}>
                        <Button
                          component="span"
                          color="primary"
                          size="small"
                          onClick={addLine}
                        >
                          Afegeix
                     </Button>
                      </Box>
                    }
                  </Grid>
                  <LinesList />
                </>
              }
              {!newInvoice &&
                <Grid container justify="flex-end" spacing={3}>
                  <Box mt={3}>
                    <InvoiceSummary />
                  </Box>
                </Grid>
              }
              {!newInvoice &&
                <Grid item xs={12} >
                  <FormGroup row>
                    <FormControlLabel control={
                      <Checkbox
                        id="charged"
                        name="charged"
                        color="primary"
                        checked={data.charged === undefined ? false : data.charged}
                        onChange={e => setData({ ...data, charged: e.target.checked })}
                        disabled={data.cancelled}
                      />
                    }
                      label="Cobrada"
                    />
                    <FormControlLabel control={
                      <Checkbox
                        id="taxes"
                        name="taxes"
                        color="primary"
                        checked={data.taxes === undefined ? false : data.taxes}
                        onChange={e => setData({ ...data, taxes: e.target.checked })}
                        disabled={data.cancelled}
                      />
                    }
                      label="Impostos pagats"
                    />
                  </FormGroup>
                </Grid>
              }
              <InvoiceFormButtons
                data={data}
                newInvoice={newInvoice}
                emitInvoice={emitInvoice}
                cancelInvoice={cancelInvoice}
                printInvoice={printInvoice}
                cancel={cancel}
                deleteInvoice={deleteInvoice}
              />
            </Grid>
          </form>
        </fieldset>
        <CustomBackdrop open={isSaving || isLoading || isEmitting || isCancelling || isDeleting} />
      </div>
    </Container >
  )
}

export default InvoiceForm
