import { FormControl, Grid, Hidden, InputAdornment, InputLabel, MenuItem, Select, TextField } from '@material-ui/core';
import { makeStyles } from '@material-ui/core/styles';
import { Search } from '@material-ui/icons';
import React, { ChangeEvent, useEffect, useState } from 'react';
import { useDispatch } from 'react-redux';
import { useHistory } from 'react-router-dom';
import api from '../../api';
import { AuthError } from '../../classes/AuthError';
import { NetworkError } from '../../classes/NetworkError';
import { searchNormalized } from '../../lib/filters';
import actions from '../../redux/actions';

const STATE_ALL = 0;
const STATE_ACTIVE = 1;
const STATE_EMITTED = 2;
const STATE_CANCELLED = 3;

const CLIENTS_ALL = '0';

type Props = {
  list: any[];
  setFiltered: Function;
}

const useStyles = makeStyles((theme) => ({
  root: {
    marginTop: theme.spacing(2)
  }
}));

const getInvoicesClients = (invoices: any[], allClients: any[]): any[] => {
  const clients = allClients.filter(client => invoices.map(invoice => invoice.client).includes(client._id));
  const output: any[] = [];
  clients.forEach(client => {
    if (!output.map(client => client._id).includes(client._id))
      output.push(client)
  });
  return output;
}

const InvoicesListFilter = ({ list, setFiltered }: Props) => {
  const dispatch = useDispatch();
  const history = useHistory();
  const classes = useStyles();

  const [search, setSearch] = useState<string>("");
  const [state, setState] = useState(STATE_ALL);
  const [selectedClient, setSelectedClient] = useState<string>(CLIENTS_ALL);
  const [clients, setClients] = useState<any[]>([]);

  const handleSearch = (e: ChangeEvent<HTMLInputElement>) => {
    setSearch(e.target.value);
  }

  const handleState = (e: ChangeEvent<any>) => {
    setState(e.target.value);
  }

  const handleClient = (e: ChangeEvent<any>) => {
    setSelectedClient(e.target.value);
  }

  const getClientName = (client: any): string => {
    return client.commercialName
      ? client.commercialName
      : [client.surnames, client.name].join(",")
  }

  // load clients list on page loaded
  useEffect(() => {
    api.getClients()
      .then(clients => setClients(clients))
      .catch(error => {
        if (error instanceof NetworkError)
          dispatch(actions.setSnackbar("No teniu connexió a Internet", "error"));
        else if (error instanceof AuthError)
          history.push("/login");
        else
          dispatch(actions.setErrors(error));
      })
  }, [history, dispatch]);

  useEffect(() => {
    setFiltered(
      list
        .filter(invoice => search === ""
          ? true
          : searchNormalized(invoice.name, search))
        .filter(invoice => state === STATE_ALL
          || (state === STATE_ACTIVE && !invoice.emitted && !invoice.cancelled)
          || (state === STATE_EMITTED && invoice.emitted && !invoice.cancelled)
          || (state === STATE_CANCELLED && invoice.cancelled)
        )
        .filter(invoice => selectedClient === CLIENTS_ALL
          || invoice.client === selectedClient)
    );
  }, [list, setFiltered, search, state, selectedClient]);

  return (
    <Grid container className={classes.root} spacing={3}>

      <Grid item xs={6} sm={4}>
        <TextField
          InputProps={{
            startAdornment: <InputAdornment position="start">
              <Search />
            </InputAdornment>
          }}
          label="Concepte"
          fullWidth
          value={search}
          onChange={handleSearch}
        />
      </Grid>

      <Grid item xs={6} sm={4}>
        <FormControl fullWidth>
          <InputLabel id="simple-select-state-label">Estat</InputLabel>
          <Select
            labelId="simple-select-state-label"
            id="client"
            fullWidth
            value={state}
            onChange={handleState}
            label="Estat"
          >
            <MenuItem value={STATE_ALL}>Totes</MenuItem>
            <MenuItem value={STATE_ACTIVE}>Actives</MenuItem>
            <MenuItem value={STATE_EMITTED}>Emeses</MenuItem>
            <MenuItem value={STATE_CANCELLED}>Anul·lades</MenuItem>
          </Select>
        </FormControl>
      </Grid>
      <Hidden xsDown>
        <Grid item xs={6} sm={4}>
          <FormControl fullWidth>
            <InputLabel id="simpe-select-client-label">Client</InputLabel>
            <Select
              labelId="simpe-select-client-label"
              id="client"
              fullWidth
              value={selectedClient}
              onChange={handleClient}
              label="Client"
            >
              <MenuItem key={CLIENTS_ALL} value={CLIENTS_ALL}>Tots</MenuItem>
              {getInvoicesClients(list, clients).map(client =>
                <MenuItem key={client._id} value={client._id}>{getClientName(client)}</MenuItem>
              )}
            </Select>
          </FormControl>
        </Grid>
      </Hidden>

    </Grid>
  )
}

export default InvoicesListFilter
