import { Box, Button, Container, Grid, InputAdornment, TextField } from '@material-ui/core';
import { makeStyles } from '@material-ui/core/styles';
import { ArrowBackIos, DeleteForever, Save } from '@material-ui/icons';
import React, { FormEvent } from 'react';
import { useSelector } from 'react-redux';
import { LINE_AMOUNT, LINE_CONCEPT_LENGTH, LINE_CONCEPT_REQUIRED, LINE_QUANTITY, LINE_VAT } from '../../constants/errors';
import { getError, hasError } from '../../lib/errors';
import { AppState } from '../../redux/reducers';
import { ErrorsState } from '../../redux/reducers/errors';
import { UiState } from '../../redux/reducers/ui';
import CustomBackdrop from '../common/CustomBackdrop';

type Props = {
  data: any;
  setData: any;
  submit: (e: FormEvent<HTMLFormElement>) => void;
  cancel: () => void;
  deleteLine: () => void;
}

const useStyles = makeStyles((theme) => ({
  paper: {
    marginTop: theme.spacing(4),
    display: 'flex',
    flexDirection: 'column',
    alignItems: 'center',
  },
  submit: {
    margin: theme.spacing(5, 0, 2),
  },
  spacer: {
    width: '100%',
    padding: theme.spacing(1),
  },
}));

const LineForm = ({ data, setData, submit, cancel, deleteLine }: Props) => {
  const classes = useStyles();
  const errors: ErrorsState = useSelector<AppState, any>(state => state.errors);
  const { isSaving, isDeleting }: UiState = useSelector<AppState, any>(state => state.ui);

  return (
    <Container component="main" maxWidth="md">
      <div className={classes.paper}>
        <fieldset disabled={isSaving || isDeleting} style={{ border: "none" }}>
          <form onSubmit={submit}>
            <Grid container spacing={3}>
              <Grid item xs={12}>
                <TextField
                  variant="outlined"
                  fullWidth
                  id="concept"
                  label="Concepte"
                  name="concept"
                  autoFocus
                  required
                  InputLabelProps={{ shrink: true }}
                  error={hasError(errors, LINE_CONCEPT_REQUIRED) || hasError(errors, LINE_CONCEPT_LENGTH)}
                  helperText={getError(errors, LINE_CONCEPT_REQUIRED) || getError(errors, LINE_CONCEPT_LENGTH)}
                  value={data.concept}
                  onChange={e => setData({ ...data, concept: e.target.value })}
                />
              </Grid>

              <Grid item xs={12} sm={3}>
                <TextField
                  type="number"
                  variant="outlined"
                  fullWidth
                  id="quantity"
                  label="Quantitat"
                  name="quantity"
                  required
                  InputLabelProps={{ shrink: true }}
                  error={hasError(errors, LINE_QUANTITY)}
                  helperText={getError(errors, LINE_QUANTITY)}
                  value={data.quantity}
                  onChange={e => setData({ ...data, quantity: e.target.value })}
                />
              </Grid>

              <Grid item xs={12} sm={6}>
                <TextField
                  type="number"
                  variant="outlined"
                  fullWidth
                  id="amount"
                  label="Import"
                  name="amount"
                  required
                  InputLabelProps={{ shrink: true }}
                  InputProps={{
                    startAdornment: (
                      <InputAdornment position="start">€</InputAdornment>
                    ),
                  }}
                  error={hasError(errors, LINE_AMOUNT)}
                  helperText={getError(errors, LINE_AMOUNT)}
                  value={data.amount}
                  onChange={e => setData({ ...data, amount: e.target.value })}
                />
              </Grid>

              <Grid item xs={12} sm={3}>
                <TextField
                  type="number"
                  variant="outlined"
                  fullWidth
                  id="vat"
                  label="IVA"
                  name="vat"
                  required
                  InputLabelProps={{ shrink: true }}
                  InputProps={{
                    startAdornment: (
                      <InputAdornment position="start">%</InputAdornment>
                    ),
                  }}
                  error={hasError(errors, LINE_VAT)}
                  helperText={getError(errors, LINE_VAT)}
                  value={data.vat}
                  onChange={e => setData({ ...data, vat: e.target.value })}
                />
              </Grid>

              <Box className={classes.spacer} />
              <Grid container spacing={3}>
                <Grid item xs={data._id ? 4 : 6}>
                  <Button
                    type="submit"
                    fullWidth
                    variant="contained"
                    color="primary"
                    startIcon={<Save />}
                    disabled={isSaving}
                  >
                    {isSaving ? "Desant" : "Desa"}
                  </Button>
                </Grid>
                <Grid item xs={data._id ? 4 : 6}>
                  <Button
                    type="button"
                    fullWidth
                    variant="contained"
                    startIcon={<ArrowBackIos />}
                    onClick={cancel}
                  >
                    Cancel·la
                  </Button>
                </Grid>
                {data._id &&
                  <Grid item xs={4}>
                    <Button
                      type="button"
                      fullWidth
                      variant="contained"
                      color="secondary"
                      startIcon={<DeleteForever />}
                      onClick={deleteLine}
                    >
                      {isDeleting ? "Eliminant" : "Elimina"}
                    </Button>
                  </Grid>
                }
              </Grid>

            </Grid>
          </form>
        </fieldset>
        <CustomBackdrop open={isSaving} />
      </div>
    </Container>
  )
}

export default LineForm
