import { Avatar, Button, Container, Grid, Link, TextField, Typography } from '@material-ui/core';
import { makeStyles } from '@material-ui/core/styles';
import { ExitToApp } from '@material-ui/icons';
import LockOutlinedIcon from '@material-ui/icons/LockOutlined';
import React, { FormEvent } from 'react';
import { useSelector } from 'react-redux';
import { EMAIL_REQUIRED, INVALID_CREDENTIALS, INVALID_EMAIL, PASSWORD_LENGTH, PASSWORD_REQUIRED } from '../../constants/errors';
import { getError, hasError } from '../../lib/errors';
import { AppState } from '../../redux/reducers';
import { ErrorsState } from '../../redux/reducers/errors';
import { UiState } from '../../redux/reducers/ui';
import CustomBackdrop from '../common/CustomBackdrop';

type Props = {
  data: any;
  setData: Function;
  submit: (e: FormEvent<HTMLFormElement>) => void;
  sendRecoveryEmail: () => Promise<void>;
  register: () => void;
}

const useStyles = makeStyles((theme) => ({
  paper: {
    marginTop: theme.spacing(8),
    display: 'flex',
    flexDirection: 'column',
    alignItems: 'center',
  },
  avatar: {
    margin: theme.spacing(1),
    backgroundColor: theme.palette.secondary.main,
  },
  form: {
    width: '100%', // Fix IE 11 issue.
    marginTop: theme.spacing(1),
  },
  submit: {
    margin: theme.spacing(3, 0, 2),
  },
}));

const LoginForm = ({ submit, data, setData, sendRecoveryEmail, register }: Props) => {
  const classes = useStyles();
  const errors: ErrorsState = useSelector<AppState, any>(state => state.errors);
  const { isLoading }: UiState = useSelector<AppState, any>(state => state.ui);

  return (
    <Container component="main" maxWidth="xs">
      <div className={classes.paper}>
        <Avatar className={classes.avatar}>
          <LockOutlinedIcon />
        </Avatar>
        <Typography component="h1" variant="h5">
          Entrar
        </Typography>

        <fieldset disabled={isLoading} style={{ border: "none" }}>
          <form onSubmit={submit} className={classes.form}>
            <TextField
              variant="outlined"
              margin="normal"
              required
              fullWidth
              id="email"
              label="Correu electrònic"
              name="email"
              autoComplete="email"
              autoFocus
              error={hasError(errors, EMAIL_REQUIRED) || hasError(errors, INVALID_EMAIL)}
              helperText={getError(errors, EMAIL_REQUIRED) || getError(errors, INVALID_EMAIL)}
              value={data.email}
              onChange={e => setData({ ...data, email: e.target.value })}
            />

            <TextField
              variant="outlined"
              margin="normal"
              required
              fullWidth
              name="password"
              label="Contrasenya"
              type="password"
              id="password"
              autoComplete="current-password"
              error={hasError(errors, PASSWORD_REQUIRED) || hasError(errors, PASSWORD_LENGTH) || hasError(errors, INVALID_CREDENTIALS)}
              helperText={getError(errors, PASSWORD_REQUIRED) || getError(errors, PASSWORD_LENGTH) || getError(errors, INVALID_CREDENTIALS)}
              value={data.password}
              onChange={e => setData({ ...data, password: e.target.value })}
            />

            <Button
              type="submit"
              fullWidth
              variant="contained"
              color="primary"
              startIcon={<ExitToApp />}
              className={classes.submit}
            >
              {isLoading ? "Autoritzant" : "Entra"}
            </Button>

            <Grid container>
              <Grid item xs>
                <Link component="button" onClick={sendRecoveryEmail} variant="body2">
                  No recordo la contrasenya
              </Link>
              </Grid>
              <Grid item>
                <Link component="button" onClick={register} variant="body2">
                  No estic registrat
              </Link>
              </Grid>
            </Grid>
          </form>
        </fieldset>
        <CustomBackdrop open={isLoading} />
      </div>
    </Container>
  )
}

export default LoginForm
