import React, { useEffect, useState } from 'react';
import { useDispatch, useSelector } from 'react-redux';
import { useHistory, useParams } from 'react-router-dom';
import api from '../api';
import { AuthError } from '../classes/AuthError';
import { NetworkError } from '../classes/NetworkError';
import ClientForm from '../components/Clients/ClientForm';
import DialogClientNotDeleted from '../components/Clients/DialogClientNotDeleted';
import DialogDeleteClient from '../components/Clients/DialogDeleteClient';
import { CLIENT_HAS_INVOICES } from '../constants/errors';
import { isNif } from '../lib/clients';
import actions from '../redux/actions';
import { AppState } from '../redux/reducers';
import { AuthState } from '../redux/reducers/auth';
import Header from '../components/common/Header';

let errorMessages: any[] = [];

const Client = () => {
  const dispatch = useDispatch();
  const history = useHistory();
  const { nif } = useParams<any>();
  const { isAuthenticated }: AuthState = useSelector<AppState, any>(state => state.auth);

  const [data, setData] = useState<any>({});

  const [newClient, setNewlient] = useState(true);

  const [openDeleteDialog, setOpenDeleteDialog] = useState<boolean>(false);
  const [openClientNotDeletedDialog, setOpenClientNotDeletedDialog] = useState<boolean>(false);

  const submit = async (e: any) => {
    e.preventDefault();
    try {
      dispatch(actions.deleteErrors());
      dispatch(actions.setSaving(true));
      const client = newClient
        ? await api.addClient(data)
        : await api.updateClient(data);
      dispatch(actions.setCurrentClient(client));
      history.goBack();
      dispatch(actions.setSnackbar("Client desat correctament"))
    } catch (error) {
      if (error instanceof NetworkError)
        dispatch(actions.setSnackbar("No teniu connexió a Internet", "error"));
      else if (error instanceof AuthError)
        return history.push("/login");
      else
        dispatch(actions.setErrors(error));
    } finally {
      dispatch(actions.setSaving(false));
    }
  }

  const askDeleteClient = () => {
    setOpenDeleteDialog(true);
  }

  const deleteClient = async () => {
    setOpenDeleteDialog(false);
    try {
      dispatch(actions.setSaving(true));
      await api.deleteClient(data._id);
      dispatch(actions.deleteClient(data._id));
      dispatch(actions.setCurrentClient({}));
      history.goBack();
      history.replace('/clients')
    } catch (error) {
      if (error instanceof NetworkError)
        dispatch(actions.setSnackbar("No teniu connexió a Internet", "error"));
      else if (error instanceof AuthError)
        return history.push("/login");
      else {
        setOpenClientNotDeletedDialog(true);
        dispatch(actions.setErrors(error));
      }
    } finally {
      dispatch(actions.setSaving(false));
    }
  }

  const cancel = () => {
    history.goBack();
  }

  // if client is set in params, load client
  useEffect(() => {
    if (isNif(nif)) {
      setNewlient(false);
      dispatch(actions.setLoading(true));
      api.getClientByNif(nif)
        .then(client => {
          setData(client);
          dispatch(actions.setCurrentClient(client));
        })
        .catch(error => {
          if (error instanceof NetworkError)
            dispatch(actions.setSnackbar("No teniu connexió a Internet", "error"));
          else if (error instanceof AuthError)
            return history.push("/login");
        })
        .finally(() => dispatch(actions.setLoading(false)));
    } else {
      dispatch(actions.setCurrentClient({}));
    }
  }, [history, dispatch, nif]);

  // set header and page title
  useEffect(() => {
    if (isNif(nif)) {
      dispatch(actions.setTitle(data.commercialName ? data.commercialName : [data.surnames, data.name].join(', ')))
    } else {
      dispatch(actions.setTitle("Nou client"));
    }
  }, [nif, data.commercialName, data.name, data.surnames, dispatch]);

  // set application language
  useEffect(() => {
    async function loadLanguage() {
      import('../languages/errors/' + process.env.REACT_APP_LANGUAGE + '.json')
        .then(result => errorMessages = result.default)
    }
    loadLanguage();
  }, []);

  return (
    <>
      {isAuthenticated &&
        <>
          <Header />
          <ClientForm
            data={data}
            setData={setData}
            newClient={newClient}
            submit={submit}
            cancel={cancel}
            deleteClient={askDeleteClient}
          />
          <DialogDeleteClient
            open={openDeleteDialog}
            setOpen={setOpenDeleteDialog}
            deleteClient={deleteClient}
          />
          <DialogClientNotDeleted
            open={openClientNotDeletedDialog}
            setOpen={setOpenClientNotDeletedDialog}
            onClose={() => null}
            message={errorMessages
              .find(error => error.code === CLIENT_HAS_INVOICES)?.message}
          />
        </>
      }
    </>
  )
}

export default Client
