import React, { useEffect, useState } from 'react';
import { useDispatch, useSelector } from 'react-redux';
import { useHistory, useParams } from 'react-router-dom';
import api from '../api';
import { AuthError } from '../classes/AuthError';
import { NetworkError } from '../classes/NetworkError';
import DialogAddLines from '../components/Invoice/DialogAddLines';
import DialogCancelInvoice from '../components/Invoice/DialogCancelInvoice';
import DialogDeleteInvoice from '../components/Invoice/DialogDeleteInvoice';
import DialogEmitInvoice from '../components/Invoice/DialogEmitInvoice';
import DialogInvoiceCancelled from '../components/Invoice/DialogInvoiceCancelled';
import DialogInvoiceEmitted from '../components/Invoice/DialogInvoiceEmitted';
import InvoiceForm from '../components/Invoice/InvoiceForm';
import { isInvoiceNumber } from '../lib/invoices';
import actions from '../redux/actions';
import { AppState } from '../redux/reducers';
import { AuthState } from '../redux/reducers/auth';
import { InvoicesState } from '../redux/reducers/invoices';
import Header from '../components/common/Header';

const Invoice = () => {
  const dispatch = useDispatch();
  const history = useHistory();
  const { number } = useParams<any>();
  const { isAuthenticated }: AuthState = useSelector<AppState, any>(state => state.auth);
  const { invoice }: InvoicesState = useSelector<AppState, any>(state => state.invoices);

  const [data, setData] = useState<any>({});
  const [clients, setClients] = useState<any[]>([]);

  const [openEmitDialog, setOpenEmitDialog] = useState<boolean>(false);
  const [openCancelDialog, setOpenCancelDialog] = useState<boolean>(false);
  const [openDeleteDialog, setOpenDeleteDialog] = useState<boolean>(false);
  const [openLinesDialog, setOpenLinesDialog] = useState<boolean>(false);
  const [openEmittedDialog, setOpenEmittedDialog] = useState<boolean>(false);
  const [openCancelledDialog, setOpenCancelledDialog] = useState<boolean>(false);

  const [newInvoice, setNewInvoice] = useState(true);

  const submit = async (e: any) => {
    e.preventDefault();
    dispatch(actions.deleteErrors());
    dispatch(actions.setSaving(true));
    try {
      const invoice: any = newInvoice
        ? await api.addInvoice(data)
        : await api.updateInvoice(data);
      dispatch(actions.setCurrentInvoice(invoice));
      if (newInvoice) {
        setOpenLinesDialog(true);
      } else {
        history.push('/invoices');
        dispatch(actions.setSnackbar("Factura desada correctament"))
      }
    } catch (error) {
      if (error instanceof NetworkError)
        dispatch(actions.setSnackbar("No teniu connexió a Internet", "error"));
      else if (error instanceof AuthError)
        return history.push("/login");
      else
        dispatch(actions.setErrors(error));
    } finally {
      dispatch(actions.setSaving(false));
    }
  }

  const addLine = () => {
    history.push('/lines/new');
  }

  const askEmitInvoice = () => {
    setOpenEmitDialog(true);
  }

  const emitInvoice = async () => {
    setOpenEmitDialog(false);
    dispatch(actions.setSaving(true));
    try {
      await api.emitInvoice(data._id);
      setOpenEmittedDialog(true);
    } catch (error) {
      if (error instanceof NetworkError)
        dispatch(actions.setSnackbar("No teniu connexió a Internet", "error"));
      else if (error instanceof AuthError)
        return history.push("/login");
      else
        dispatch(actions.setErrors(error));
    } finally {
      dispatch(actions.setSaving(false));
    }
  }

  const askCancelInvoice = () => {
    setOpenCancelDialog(true);
  }

  const cancelInvoice = async () => {
    setOpenCancelDialog(false);
    dispatch(actions.setSaving(true));
    try {
      await api.cancelInvoice(data._id);
      setOpenCancelledDialog(true);
    } catch (error) {
      if (error instanceof NetworkError)
        dispatch(actions.setSnackbar("No teniu connexió a Internet", "error"));
      else if (error instanceof AuthError)
        return history.push("/login");
      else
        dispatch(actions.setErrors(error));
    } finally {
      dispatch(actions.setSaving(false));
    }
  }

  const printInvoice = () => {
    history.push('/print/' + invoice.number);
  }

  const askDeleteInvoice = () => {
    setOpenDeleteDialog(true);
  }

  const deleteInvoice = async () => {
    setOpenDeleteDialog(false);
    dispatch(actions.setSaving(true));
    try {
      await api.deleteInvoice(data._id);
      dispatch(actions.deleteInvoice(data._id));
      dispatch(actions.setCurrentInvoice({}));
      history.goBack();
      history.replace('/invoices');
    } catch (error) {
      if (error instanceof NetworkError)
        dispatch(actions.setSnackbar("No teniu connexió a Internet", "error"));
      else if (error instanceof AuthError)
        return history.push("/login");
      else
        dispatch(actions.setErrors(error));
    } finally {
      dispatch(actions.setSaving(false));
    }
  }

  const cancel = () => {
    history.goBack();
  }

  // load clients list on page loaded
  useEffect(() => {
    api.getClients()
      .then(clients => setClients(clients))
      .catch(error => {
        if (error instanceof NetworkError)
          dispatch(actions.setSnackbar("No teniu connexió a Internet", "error"));
        else if (error instanceof AuthError)
          history.push("/login");
        else
          dispatch(actions.setErrors(error));
      })
  }, [history, dispatch]);

  // if invoice number is set in params, load invoice
  useEffect(() => {
    if (isInvoiceNumber(number)) {
      setNewInvoice(false);
      dispatch(actions.setLoading(true));
      api.getInvoice(number)
        .then(invoice => {
          setData({ ...invoice, date: invoice.date.substr(0, 10) });
          dispatch(actions.setCurrentInvoice(invoice));
        })
        .catch(error => {
          if (error instanceof NetworkError)
            dispatch(actions.setSnackbar("No teniu connexió a Internet", "error"));
          else if (error instanceof AuthError)
            return history.push("/login");
        })
        .finally(() => dispatch(actions.setLoading(false)));
    } else {
      dispatch(actions.setCurrentInvoice({}));
    }
  }, [dispatch, history, number]);

  // change document title
  useEffect(() => {
    dispatch(actions.setTitle(data.number ? `Factura ${data.number}` : "Nova factura"));
  }, [data.number, dispatch]);

  return (
    <>
      {isAuthenticated &&
        <>
          <Header />
          <InvoiceForm
            data={data}
            setData={setData}
            clients={clients}
            newInvoice={newInvoice}
            emitInvoice={askEmitInvoice}
            cancelInvoice={askCancelInvoice}
            printInvoice={printInvoice}
            deleteInvoice={askDeleteInvoice}
            addLine={addLine}
            submit={submit}
            cancel={cancel}
          />
          <DialogEmitInvoice
            open={openEmitDialog}
            setOpen={setOpenEmitDialog}
            emitInvoice={emitInvoice}
          />
          <DialogCancelInvoice
            open={openCancelDialog}
            setOpen={setOpenCancelDialog}
            cancelInvoice={cancelInvoice}
          />
          <DialogDeleteInvoice
            open={openDeleteDialog}
            setOpen={setOpenDeleteDialog}
            deleteInvoice={deleteInvoice}
          />
          <DialogAddLines
            open={openLinesDialog}
            setOpen={setOpenLinesDialog}
            addLines={addLine}
            dontAddLines={() => history.goBack()}
          />
          <DialogInvoiceEmitted
            open={openEmittedDialog}
            setOpen={setOpenEmittedDialog}
            onClose={() => history.goBack()}
          />
          <DialogInvoiceCancelled
            open={openCancelledDialog}
            setOpen={setOpenCancelledDialog}
            onClose={() => history.goBack()}
          />
        </>
      }
    </>
  )
}

export default Invoice
