import React, { useEffect, useState } from 'react';
import { useDispatch, useSelector } from 'react-redux';
import { useHistory, useParams } from 'react-router-dom';
import api from '../api';
import { AuthError } from '../classes/AuthError';
import { NetworkError } from '../classes/NetworkError';
import DialogDeleteLine from '../components/Lines/DialogDeleteLine';
import LineForm from '../components/Lines/LineForm';
import actions from '../redux/actions';
import { AppState } from '../redux/reducers';
import { InvoicesState } from '../redux/reducers/invoices';
import { LinesState } from '../redux/reducers/lines';
import Header from '../components/common/Header';
import { AuthState } from '../redux/reducers/auth';

const Line = () => {
  const dispatch = useDispatch();
  const history = useHistory();
  const { id } = useParams<any>();
  const { isAuthenticated }: AuthState = useSelector<AppState, any>(state => state.auth);
  const { lines }: LinesState = useSelector<AppState, any>(state => state.lines);
  const { invoice }: InvoicesState = useSelector<AppState, any>(state => state.invoices);

  const [line, setLine] = useState<any>({});

  const [openDeleteDialog, setOpenDeleteDialog] = useState<boolean>(false);

  const askDeleteLine = () => {
    setOpenDeleteDialog(true);
  }

  const deleteLine = async () => {
    setOpenDeleteDialog(false);
    dispatch(actions.setSaving(true));
    try {
      await api.deleteLine(line._id);
      dispatch(actions.deleteLine(line._id));
      history.goBack();
      history.replace(history.location);
    } catch (error) {
      if (error instanceof NetworkError)
        dispatch(actions.setSnackbar("No teniu connexió a Internet", "error"));
      else if (error instanceof AuthError)
        return history.push("/login");
      else
        dispatch(actions.setErrors(error));
    } finally {
      dispatch(actions.setSaving(false));
    }
  }

  const submit = async (e: any) => {
    e.preventDefault();
    try {
      dispatch(actions.deleteErrors());
      dispatch(actions.setSaving(true));
      if (line._id) {
        await api.updateLine(line);
      } else {
        if (invoice._id) await api.addLine(invoice._id, line);
      }
      history.replace('/invoices/' + invoice.number);
      dispatch(actions.setSnackbar("Línia desada correctament"))
    } catch (error) {
      if (error instanceof NetworkError)
        dispatch(actions.setSnackbar("No teniu connexió a Internet", "error"));
      else if (error instanceof AuthError)
        return history.push("/login");
      else
        dispatch(actions.setErrors(error));
    } finally {
      dispatch(actions.setSaving(false));
    }
  }

  const cancel = () => {
    history.goBack();
  }

  // if line id is set in params, load line
  useEffect(() => {
    setLine(lines.find(line => line._id === id) || {});
  }, [lines, id]);

  // set header and page title
  useEffect(() => {
    dispatch(actions.setTitle('Línia de factura'));
  }, [dispatch]);

  return (
    <>
      {isAuthenticated &&
        <>
          <Header />
          <LineForm
            data={line}
            setData={setLine}
            submit={submit}
            cancel={cancel}
            deleteLine={askDeleteLine}
          />
          <DialogDeleteLine
            open={openDeleteDialog}
            setOpen={setOpenDeleteDialog}
            deleteLine={deleteLine}
          />
        </>
      }
    </>
  )
}

export default Line
